unit poteze;

interface

uses graphics, sysUtils, windows, math;

const
  utezKrogi=25;
  utezKrogiElipse=0.10;
  utezCrte=0.5;
  utezElipse=0.9;
  delezOdsek=10;
  maxBezier=100;

type
  TipPoteze=(TPnic, TPcrta, TPkrog);
  Poteza=class
    tocke, lomisca: array of TPoint;
    lomiscaBezier: array [0..maxBezier] of TPoint;
    zadnjaTocka, stTock, zadnjeLomisce, stLomisc: integer;
    tip: TipPoteze;
    ime: string;
    sredisce: TPoint;
    polmer, polmerX, polmerY: real;
    constructor create;
    procedure dodajTocko (x, y: integer);
    procedure narisi (kam: TCanvas; barva: TColor; crte, krogi: integer; bezier, elipse: boolean; mreza, koti: integer; odZac: boolean);
  end;

implementation

function razdalja (t1, t2: TPoint): real;
begin
  razdalja:=sqrt (sqr (t2.x-t1.x)+sqr (t2.y-t1.y));
end;

function kot (t1, t2: TPoint): real;
  var
    zacR: real;
begin
  if t2.x<>t1.x then
    zacR:=arcTan (abs (t2.y-t1.y)/abs (t2.x-t1.x))
  else
    zacR:=pi/2;
  if (t2.x>=t1.x) and (t2.y>=t1.y) then
    kot:=zacR
  else
    if (t2.x>=t1.x) and (t2.y<t1.y) then
      kot:=2*pi-zacR
    else
      if (t2.x<t1.x) and (t2.y<t1.y) then
        kot:=pi+zacR
      else
        if (t2.x<t1.x) and (t2.y>=t1.y) then
          kot:=pi-zacR
        else
          kot:=10;
end;

function omrezi (kaj, korak: integer): integer;
  var
    manj: integer;
begin
  if korak>0  then begin
    manj:=kaj div korak;
    if kaj/korak-manj<=0.5 then
      omrezi:=manj*korak
    else
      omrezi:=(manj+1)*korak;
  end else
    omrezi:=kaj;
end;

function okoti (odT, doT: TPoint; korak: real): TPoint;
  var
    kotIzvirne, manj, dolzina: real;
    zacTP: TPoint;
begin
  dolzina:=razdalja (odT, doT);
  kotIzvirne:=kot (odT, doT);
  manj:=floor (kotIzvirne/korak);
  if kotIzvirne/korak-manj<=0.5 then begin
    zacTP.x:=round (odT.x+dolzina*cos (manj*korak));
    zacTP.y:=round (odT.y+dolzina*sin (manj*korak));
  end else begin
    zacTP.x:=round (odT.x+dolzina*cos ((manj+1)*korak));
    zacTP.y:=round (odT.y+dolzina*sin ((manj+1)*korak));
  end;
  okoti:=zacTP;
end;

constructor Poteza.create;
begin
  stTock:=100;
  setLength (tocke, stTock);
  zadnjaTocka:=-1;
  tip:=TPnic;
  ime:='Neobdelana poteza';
  stLomisc:=50;
  setLength (lomisca, stLomisc);
  zadnjeLomisce:=-1;
end;

procedure Poteza.dodajTocko (x, y: integer);
begin
  inc (zadnjaTocka);
  if (zadnjaTocka>=stTock) then begin
    stTock:=stTock+100;
    setLength (tocke, stTock);
  end;
  tocke [zadnjaTocka].x:=x;
  tocke [zadnjaTocka].y:=y;
end;

procedure Poteza.narisi (kam: TCanvas; barva: TColor; crte, krogi: integer; bezier, elipse: boolean; mreza, koti: integer; odZac: boolean);
  var
    i, delez, odsek1, odsek2, odsek3, odsek4, kvadrant1, kvadrant2, kvadrant3, kvadrant4, zacetek, konec, st, omrezenPolmer: integer;
    maxX, maxY, mejniKot, trenutniKot, trenutniPolmer, minPolmer, maxPolmer, a, b, c, trenutnaRazdalja, delitelj: real;
    zacetnaTocka, koncnaTocka: TPoint;
begin
  kam.pen.color:=barva;
  if (tip=TPnic) and (krogi>0) and (zadnjaTocka>0) then begin
    sredisce.x:=0;
    sredisce.y:=0;
    for i:=1 to zadnjaTocka do begin
      sredisce.x:=sredisce.x+tocke [i].x;
      sredisce.y:=sredisce.y+tocke [i].y;
    end;
    sredisce.x:=round (sredisce.x/zadnjaTocka);
    sredisce.y:=round (sredisce.y/zadnjaTocka);
    if elipse then begin
      maxX:=0;
      maxY:=0;
      for i:=1 to zadnjaTocka do begin
        trenutniPolmer:=abs (sredisce.x-tocke [i].x);
        if trenutniPolmer>maxX then
          maxX:=trenutniPolmer;
        trenutniPolmer:=abs (sredisce.y-tocke [i].y);
        if trenutniPolmer>maxY then
          maxY:=trenutniPolmer;
      end;
      delez:=round ((zadnjaTocka+1)/(4+krogi*utezKrogiElipse));
      mejniKot:=arcTan (maxY/maxX);
      odsek1:=0;
      odsek2:=0;
      odsek3:=0;
      odsek4:=0;
      kvadrant1:=0;
      kvadrant2:=0;
      kvadrant3:=0;
      kvadrant4:=0;
      for i:=1 to zadnjaTocka do begin
        trenutniKot:=kot (sredisce, tocke [i]);
        if (trenutniKot>mejniKot) and (trenutniKot<pi-mejniKot) then
          inc (odsek1)
        else
          if (trenutniKot>pi-mejniKot) and (trenutniKot<pi+mejniKot) then
            inc (odsek2)
          else
            if (trenutniKot>pi+mejniKot) and (trenutniKot<2*pi-mejniKot) then
              inc (odsek3)
            else
              if (trenutniKot>2*pi-mejniKot) or (trenutniKot<mejniKot) then
                inc (odsek4);
        if (tocke [i].x>=sredisce.x) and (tocke [i].y>=sredisce.y) then
          inc (kvadrant1)
        else
          if (tocke [i].x>=sredisce.x) and (tocke [i].y<sredisce.y) then
            inc (kvadrant2)
          else
            if (tocke [i].x<sredisce.x) and (tocke [i].y<sredisce.y) then
              inc (kvadrant3)
            else
              if (tocke [i].x<sredisce.x) and (tocke [i].y>=sredisce.y) then
                inc (kvadrant4);
      end;
      if (odsek1>delez) and (odsek2>delez) and (odsek3>delez) and (odsek4>delez) and (kvadrant1>delez) and (kvadrant2>delez) and (kvadrant3>delez) and (kvadrant4>delez) then begin
        tip:=TPkrog;
        polmerX:=maxX*utezElipse;
        polmerY:=maxY*utezElipse;
      end;
    end else begin
      minPolmer:=1000;
      maxPolmer:=0;
      polmer:=0;
      for i:=1 to zadnjaTocka do begin
        trenutniPolmer:=razdalja (sredisce, tocke [i]);
        if trenutniPolmer<minPolmer then
          minPolmer:=trenutniPolmer;
        if trenutniPolmer>maxPolmer then
          maxPolmer:=trenutniPolmer;
        polmer:=polmer+trenutniPolmer;
      end;
      polmer:=polmer/zadnjaTocka;
      if maxPolmer/(minPolmer+0.01)*utezKrogi<krogi then
        tip:=TPkrog;
    end;
  end;
  if (tip=TPnic) and (crte>0) and (zadnjaTocka>1) then begin
    repeat
      zadnjeLomisce:=0;
      lomisca [zadnjeLomisce]:=tocke [0];
      zacetek:=0;
      konec:=2;
      repeat
        zacetnaTocka:=tocke [zacetek];
        koncnaTocka:=tocke [konec];
        if zacetnaTocka.x=koncnaTocka.x then begin
          a:=1;
          b:=0;
          c:=-zacetnaTocka.x;
        end else
          if zacetnaTocka.y=koncnaTocka.y then begin
            a:=0;
            b:=1;
            c:=-zacetnaTocka.y;
          end else begin
            a:=(koncnaTocka.y-zacetnaTocka.y)/(koncnaTocka.x-zacetnaTocka.x);
            b:=-1;
            c:=zacetnaTocka.y-a*zacetnaTocka.x;
          end;
        delitelj:=sqrt (sqr (a)+sqr (b));
        for i:=zacetek+1 to konec do begin
          trenutnaRazdalja:=abs ((a*tocke [i].x+b*tocke [i].y+c)/delitelj);
          if trenutnaRazdalja>crte*utezCrte then begin
            inc (zadnjeLomisce);
            if zadnjeLomisce>=stLomisc then begin
              stLomisc:=stLomisc+50;
              setLength (lomisca, stLomisc);
            end;
            lomisca [zadnjeLomisce]:=tocke [konec-1];
            zacetek:=konec-1;
            break;
          end;
        end;
        inc (konec);
      until konec>zadnjaTocka;
      inc (zadnjeLomisce);
      if zadnjeLomisce>=stLomisc then begin
        stLomisc:=stLomisc+50;
        setLength (lomisca, stLomisc);
      end;
      lomisca [zadnjeLomisce]:=tocke [zadnjaTocka];
      tip:=TPcrta;
      inc (crte);
    until not bezier or (zadnjeLomisce=1) or (zadnjeLomisce mod 3=0);
  end;
  case tip of
    TPnic: begin
      kam.moveTo (tocke [0].x, tocke [0].y);
      for i:=1 to zadnjaTocka do
        kam.lineTo (tocke [i].x, tocke [i].y);
      ime:='Neobdelana poteza';
    end;
    TPcrta: begin
      if bezier and (zadnjeLomisce<=maxBezier) and (zadnjeLomisce>1) then begin
        for i:=0 to zadnjeLomisce do begin
          lomiscaBezier [i].x:=omrezi (lomisca [i].x, mreza);
          lomiscaBezier [i].y:=omrezi (lomisca [i].y, mreza);
        end;
        kam.polyBezier (slice (lomiscaBezier, zadnjeLomisce+1));
        st:=zadnjeLomisce div 3;
        ime:=intToStr (st);
        case st of
          1: ime:=ime+' Bezierjeva krivulja';
          2: ime:=ime+' Bezierjevi kriuvlji';
          3, 4: ime:=ime+' Bezierjeve krivulje';
          else ime:=ime+' Bezierjevih krivulj';
        end;
      end else begin
        if (zadnjeLomisce=1) and (koti>0) then
          if odZac then begin
            kam.moveTo (omrezi (lomisca [0].x, mreza), omrezi (lomisca [0].y, mreza));
            koncnaTocka:=okoti (lomisca [0], lomisca [1], degToRad (koti));
            kam.lineTo (omrezi (koncnaTocka.x, mreza), omrezi (koncnaTocka.y, mreza));
          end else begin
            kam.moveTo (omrezi (lomisca [1].x, mreza), omrezi (lomisca [1].y, mreza));
            koncnaTocka:=okoti (lomisca [1], lomisca [0], degToRad (koti));
            kam.lineTo (omrezi (koncnaTocka.x, mreza), omrezi (koncnaTocka.y, mreza));
          end
        else begin
          kam.moveTo (omrezi (lomisca [0].x, mreza), omrezi (lomisca [0].y, mreza));
          for i:=1 to zadnjeLomisce do
            kam.lineTo (omrezi (lomisca [i].x, mreza), omrezi (lomisca [i].y, mreza));
        end;
        ime:=intToStr (zadnjeLomisce-1)+'x lomljena crta';
      end;
    end;
    TPkrog: begin
      kam.brush.style:=bsClear;
      if elipse then begin
        kam.ellipse (omrezi (round (sredisce.x-polmerX), mreza), omrezi (round (sredisce.y-polmerY), mreza), omrezi (round (sredisce.x+polmerX), mreza), omrezi (round (sredisce.y+polmerY), mreza));
        ime:='Elipsa';
      end else begin
        if mreza=0 then
          kam.ellipse (round (sredisce.x-polmer), round (sredisce.y-polmer), round (sredisce.x+polmer), round (sredisce.y+polmer))
        else begin
          omrezenPolmer:=omrezi (round (polmer), mreza);
          kam.ellipse (omrezi (round (sredisce.x), mreza)-omrezenPolmer, omrezi (round (sredisce.y), mreza)-omrezenPolmer, omrezi (round (sredisce.x), mreza)+omrezenPolmer, omrezi (round (sredisce.y), mreza)+omrezenPolmer);
        end;
        ime:='Krog';
      end;
    end;
  end;
end;

end.
